==> carillon_map_box-0.0.5.tar.gz: android/src/main/java/com/example/carillon_map_box/NavigationActivity.kt <==
package com.example.carillon_map_box

import android.Manifest
import android.content.pm.PackageManager
import android.content.res.Resources
import android.location.Location
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import androidx.core.content.ContextCompat
import com.mapbox.api.directions.v5.models.DirectionsRoute
import com.mapbox.api.directions.v5.models.RouteOptions
import com.mapbox.geojson.Point
import com.mapbox.maps.EdgeInsets
import com.mapbox.maps.MapView
import com.mapbox.maps.MapboxMap
import com.mapbox.maps.Style
import com.mapbox.maps.extension.style.style
import com.mapbox.maps.plugin.LocationPuck2D
import com.mapbox.maps.plugin.animation.camera
import com.mapbox.maps.plugin.locationcomponent.location
import com.mapbox.navigation.base.extensions.applyDefaultNavigationOptions
import com.mapbox.navigation.base.extensions.applyLanguageAndVoiceUnitOptions
import com.mapbox.navigation.base.formatter.DistanceFormatterOptions
import com.mapbox.navigation.base.options.NavigationOptions
import com.mapbox.navigation.base.route.RouterCallback
import com.mapbox.navigation.base.route.RouterFailure
import com.mapbox.navigation.base.route.RouterOrigin
import com.mapbox.navigation.base.trip.model.RouteProgress
import com.mapbox.navigation.core.MapboxNavigation
import com.mapbox.navigation.core.MapboxNavigationProvider
import com.mapbox.navigation.core.directions.session.RoutesObserver
import com.mapbox.navigation.core.directions.session.RoutesUpdatedResult
import com.mapbox.navigation.core.trip.session.LocationMatcherResult
import com.mapbox.navigation.core.trip.session.LocationObserver
import com.mapbox.navigation.core.trip.session.RouteProgressObserver
import com.mapbox.navigation.ui.maps.camera.NavigationCamera
import com.mapbox.navigation.ui.maps.camera.data.MapboxNavigationViewportDataSource
import com.mapbox.navigation.ui.maps.camera.transition.NavigationCameraTransitionOptions
import com.mapbox.navigation.ui.maps.location.NavigationLocationProvider
import com.mapbox.navigation.ui.maps.route.line.api.MapboxRouteLineApi
import com.mapbox.navigation.ui.maps.route.line.api.MapboxRouteLineView
import com.mapbox.navigation.ui.maps.route.line.model.MapboxRouteLineOptions
import com.mapbox.navigation.ui.maps.route.line.model.RouteLine
import com.mapbox.navigation.ui.tripprogress.view.MapboxTripProgressView
import com.mapbox.navigation.ui.tripprogress.api.MapboxTripProgressApi
import com.mapbox.navigation.ui.tripprogress.model.DistanceRemainingFormatter
import com.mapbox.navigation.ui.tripprogress.model.EstimatedTimeToArrivalFormatter
import com.mapbox.navigation.ui.tripprogress.model.TimeRemainingFormatter
import com.mapbox.navigation.ui.tripprogress.model.TripProgressUpdateFormatter

class NavigationActivity : AppCompatActivity() {
    private var PERMISSION_REQUEST_CODE: Int = 367
    private val navigationLocationProvider = NavigationLocationProvider()
    private lateinit var mapboxNavigation : MapboxNavigation
    private lateinit var tripProgressView : MapboxTripProgressView
    private lateinit var mapView: MapView
    private lateinit var viewportDataSource : MapboxNavigationViewportDataSource
    private lateinit var navigationCamera : NavigationCamera
    private lateinit var routeLineApi: MapboxRouteLineApi
    private lateinit var routeLineView: MapboxRouteLineView

    private val tripProgressFormatter: TripProgressUpdateFormatter by lazy {
        val distanceFormatterOptions =
                DistanceFormatterOptions.Builder(this).build()

        TripProgressUpdateFormatter.Builder(this)
                .distanceRemainingFormatter(DistanceRemainingFormatter(distanceFormatterOptions))
                .timeRemainingFormatter(TimeRemainingFormatter(this))
                .estimatedTimeToArrivalFormatter(EstimatedTimeToArrivalFormatter(this))
                .build()
    }

    private val tripProgressApi: MapboxTripProgressApi by lazy {
        MapboxTripProgressApi(tripProgressFormatter)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_navigation)

        tripProgressView = findViewById(R.id.tripProgressView)
        mapView = findViewById(R.id.mapView)

        mapView.location.apply {
            this.locationPuck = LocationPuck2D(
                    bearingImage = ContextCompat.getDrawable(
                            this@NavigationActivity,
                            R.drawable.mapbox_navigation_puck_icon
                    )
            )
            setLocationProvider(navigationLocationProvider)
            enabled = true
        }

        viewportDataSource = MapboxNavigationViewportDataSource(mapView.getMapboxMap())
        navigationCamera = NavigationCamera(mapView.getMapboxMap(), mapView.camera, viewportDataSource)

        viewportDataSource.overviewPadding = overviewPadding
        viewportDataSource.followingPadding = followingPadding

        val accessToken = this.applicationContext.getString(this.applicationContext.resources.getIdentifier("mapbox_access_token", "string", this.applicationContext.packageName))

        val plaisir = Point.fromLngLat(1.96, 48.82)
        var destination = Point.fromLngLat(2.25, 48.8)

        val navigationOptions = NavigationOptions.Builder(this.applicationContext)
                .accessToken(accessToken)
                .build()
        val routeOptions = RouteOptions.builder()
                .applyDefaultNavigationOptions()
                .coordinatesList(listOf(plaisir, destination))
                .build()

        val mapboxRouteLineOptions = MapboxRouteLineOptions.Builder(this)
                .withRouteLineBelowLayerId("road-label")
                .build()

        routeLineApi = MapboxRouteLineApi(mapboxRouteLineOptions)
        routeLineView = MapboxRouteLineView(mapboxRouteLineOptions)

        navigationCamera.requestNavigationCameraToFollowing()

        mapboxNavigation = MapboxNavigationProvider.create(navigationOptions)
        checkPermission()
        mapboxNavigation.startTripSession()

        mapView.getMapboxMap().loadStyleUri(Style.MAPBOX_STREETS)

        mapboxNavigation.requestRoutes(routeOptions, routeRequestCallBack)
    }

    private val routeRequestCallBack = object : RouterCallback {
        override fun onRoutesReady(routes: List<DirectionsRoute>, routerOrigin: RouterOrigin) {
            mapboxNavigation.setRoutes(routes)
            navigationCamera.requestNavigationCameraToFollowing()
        }

        override fun onCanceled(routeOptions: RouteOptions, routerOrigin: RouterOrigin) {
            TODO("Not yet implemented")
        }

        override fun onFailure(reasons: List<RouterFailure>, routeOptions: RouteOptions) {
            TODO("Not yet implemented")
        }
    }

    private val locationObserver = object : LocationObserver {
        var firstLocationUpdateReceived = false

        override fun onNewRawLocation(rawLocation: Location) {
            viewportDataSource.onLocationChanged(rawLocation)
            viewportDataSource.evaluate()
        }

        override fun onNewLocationMatcherResult(locationMatcherResult: LocationMatcherResult) {
            navigationLocationProvider.changePosition(
                    location = locationMatcherResult.enhancedLocation,
                    keyPoints = locationMatcherResult.keyPoints
            )
            viewportDataSource.onLocationChanged(locationMatcherResult.enhancedLocation)
            viewportDataSource.evaluate()

            if (!firstLocationUpdateReceived) {
                firstLocationUpdateReceived = true
                navigationCamera.requestNavigationCameraToOverview(
                        stateTransitionOptions = NavigationCameraTransitionOptions.Builder()
                                .maxDuration(0) // instant transition
                                .build()
                )
            }
        }

    }

    private val routeObserver = RoutesObserver { result: RoutesUpdatedResult ->
        if (result.routes.isNotEmpty()) {
            val routeLines = result.routes.map { RouteLine(it, null) }
            routeLineApi.setRoutes(
                    routeLines
            ) { value ->
                mapView.getMapboxMap().getStyle()?.apply {
                    routeLineView.renderRouteDrawData(this, value)
                }
            }

            viewportDataSource.onRouteChanged(result.routes.first())
            viewportDataSource.evaluate()
        } else {
            val style = mapView.getMapboxMap().getStyle()
            if (style != null) {
                routeLineApi.clearRouteLine { value ->
                    routeLineView.renderClearRouteLineValue(
                            style,
                            value
                    )
                }
            }

            viewportDataSource.clearRouteData()
            viewportDataSource.evaluate()
        }
    }

    private val routeProgressObserver = RouteProgressObserver { routeProgress ->
        viewportDataSource.onRouteProgressChanged(routeProgress)
        viewportDataSource.evaluate()
        tripProgressApi.getTripProgress(routeProgress).let { update ->
            tripProgressView.render(update)
        }
    }

    private fun checkPermission() {
        val hasperm = this.checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION)
        if (hasperm != PackageManager.PERMISSION_GRANTED) {
            this.requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), PERMISSION_REQUEST_CODE)
        }
    }

    override fun onStart() {
        super.onStart()
        mapboxNavigation.registerRouteProgressObserver(routeProgressObserver)
        mapboxNavigation.registerRoutesObserver(routeObserver)
        mapboxNavigation.registerLocationObserver(locationObserver)
    }

    override fun onStop() {
        super.onStop()
        mapboxNavigation.unregisterRoutesObserver(routeObserver)
        mapboxNavigation.unregisterRouteProgressObserver(routeProgressObserver)
        mapboxNavigation.unregisterLocationObserver(locationObserver)
    }

    override fun onDestroy() {
        super.onDestroy()
        routeLineApi.cancel()
        routeLineView.cancel()
        MapboxNavigationProvider.destroy()
    }

    private val pixelDensity = Resources.getSystem().displayMetrics.density
    private val overviewPadding: EdgeInsets by lazy {
        EdgeInsets(
                140.0 * pixelDensity,
                40.0 * pixelDensity,
                120.0 * pixelDensity,
                40.0 * pixelDensity
        )
    }
    private val followingPadding: EdgeInsets by lazy {
        EdgeInsets(
                180.0 * pixelDensity,
                40.0 * pixelDensity,
                150.0 * pixelDensity,
                40.0 * pixelDensity
        )
    }
}

==> carillon_map_box-0.0.6.tar.gz: android/src/main/java/com/example/carillon_map_box/NavigationActivity.kt <==
package com.example.carillon_map_box

import android.Manifest
import android.content.Context
import android.content.pm.PackageManager
import android.content.res.Resources
import android.location.Location
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.View.*
import android.widget.ImageButton
import android.widget.Toast
import androidx.core.content.ContextCompat
import com.mapbox.api.directions.v5.DirectionsCriteria
import com.mapbox.api.directions.v5.models.DirectionsRoute
import com.mapbox.api.directions.v5.models.RouteOptions
import com.mapbox.geojson.Point
import com.mapbox.maps.EdgeInsets
import com.mapbox.maps.MapView
import com.mapbox.maps.MapboxMap
import com.mapbox.maps.Style
import com.mapbox.maps.extension.style.style
import com.mapbox.maps.plugin.LocationPuck2D
import com.mapbox.maps.plugin.animation.camera
import com.mapbox.maps.plugin.locationcomponent.location
import com.mapbox.navigation.base.extensions.applyDefaultNavigationOptions
import com.mapbox.navigation.base.extensions.applyLanguageAndVoiceUnitOptions
import com.mapbox.navigation.base.formatter.DistanceFormatterOptions
import com.mapbox.navigation.base.options.NavigationOptions
import com.mapbox.navigation.base.route.RouterCallback
import com.mapbox.navigation.base.route.RouterFailure
import com.mapbox.navigation.base.route.RouterOrigin
import com.mapbox.navigation.base.trip.model.RouteLegProgress
import com.mapbox.navigation.base.trip.model.RouteProgress
import com.mapbox.navigation.core.MapboxNavigation
import com.mapbox.navigation.core.MapboxNavigationProvider
import com.mapbox.navigation.core.arrival.ArrivalObserver
import com.mapbox.navigation.core.directions.session.RoutesObserver
import com.mapbox.navigation.core.directions.session.RoutesUpdatedResult
import com.mapbox.navigation.core.formatter.MapboxDistanceFormatter
import com.mapbox.navigation.core.trip.session.LocationMatcherResult
import com.mapbox.navigation.core.trip.session.LocationObserver
import com.mapbox.navigation.core.trip.session.RouteProgressObserver
import com.mapbox.navigation.ui.maneuver.api.MapboxManeuverApi
import com.mapbox.navigation.ui.maneuver.api.RoadShieldCallback
import com.mapbox.navigation.ui.maneuver.model.Maneuver
import com.mapbox.navigation.ui.maneuver.model.RoadShield
import com.mapbox.navigation.ui.maneuver.model.RoadShieldError
import com.mapbox.navigation.ui.maneuver.view.MapboxLaneGuidance
import com.mapbox.navigation.ui.maneuver.view.MapboxManeuverView
import com.mapbox.navigation.ui.maps.camera.NavigationCamera
import com.mapbox.navigation.ui.maps.camera.data.FollowingFrameOptions
import com.mapbox.navigation.ui.maps.camera.data.MapboxNavigationViewportDataSource
import com.mapbox.navigation.ui.maps.camera.lifecycle.NavigationBasicGesturesHandler
import com.mapbox.navigation.ui.maps.camera.state.NavigationCameraState
import com.mapbox.navigation.ui.maps.camera.state.NavigationCameraStateChangedObserver
import com.mapbox.navigation.ui.maps.camera.transition.NavigationCameraTransitionOptions
import com.mapbox.navigation.ui.maps.camera.view.MapboxRecenterButton
import com.mapbox.navigation.ui.maps.camera.view.MapboxRouteOverviewButton
import com.mapbox.navigation.ui.maps.location.NavigationLocationProvider
import com.mapbox.navigation.ui.maps.route.arrow.api.MapboxRouteArrowApi
import com.mapbox.navigation.ui.maps.route.arrow.api.MapboxRouteArrowView
import com.mapbox.navigation.ui.maps.route.arrow.model.RouteArrowOptions
import com.mapbox.navigation.ui.maps.route.line.api.MapboxRouteLineApi
import com.mapbox.navigation.ui.maps.route.line.api.MapboxRouteLineView
import com.mapbox.navigation.ui.maps.route.line.model.MapboxRouteLineOptions
import com.mapbox.navigation.ui.maps.route.line.model.RouteLine
import com.mapbox.navigation.ui.tripprogress.view.MapboxTripProgressView
import com.mapbox.navigation.ui.tripprogress.api.MapboxTripProgressApi
import com.mapbox.navigation.ui.tripprogress.model.DistanceRemainingFormatter
import com.mapbox.navigation.ui.tripprogress.model.EstimatedTimeToArrivalFormatter
import com.mapbox.navigation.ui.tripprogress.model.TimeRemainingFormatter
import com.mapbox.navigation.ui.tripprogress.model.TripProgressUpdateFormatter
import java.util.*

class NavigationActivity : AppCompatActivity() {
    private var PERMISSION_REQUEST_CODE: Int = 367
    private val navigationLocationProvider = NavigationLocationProvider()
    private lateinit var mapboxNavigation : MapboxNavigation
    private lateinit var tripProgressView : MapboxTripProgressView
    private lateinit var maneuverView: MapboxManeuverView
    private lateinit var mapView: MapView
    private lateinit var viewportDataSource : MapboxNavigationViewportDataSource
    private lateinit var navigationCamera : NavigationCamera
    private lateinit var routeLineApi: MapboxRouteLineApi
    private lateinit var routeLineView: MapboxRouteLineView
    private lateinit var routeArrowApi: MapboxRouteArrowApi
    private lateinit var routeArrowView: MapboxRouteArrowView
    private lateinit var routeArrowOptions: RouteArrowOptions
    private lateinit var carillonLocation: List<Map<String, Objects>>
    private lateinit var carillonOptions: Map<String, Objects>
    private lateinit var overviewButton : MapboxRouteOverviewButton
    private lateinit var recenterButton: MapboxRecenterButton
    private lateinit var closeButton: ImageButton
    private var points: MutableList<Point> = mutableListOf()

    private val tripProgressFormatter: TripProgressUpdateFormatter by lazy {
        val distanceFormatterOptions =
                DistanceFormatterOptions.Builder(this).build()

        TripProgressUpdateFormatter.Builder(this)
                .distanceRemainingFormatter(DistanceRemainingFormatter(distanceFormatterOptions))
                .timeRemainingFormatter(TimeRemainingFormatter(this))
                .estimatedTimeToArrivalFormatter(EstimatedTimeToArrivalFormatter(this))
                .build()
    }

    private val tripProgressApi: MapboxTripProgressApi by lazy {
        MapboxTripProgressApi(tripProgressFormatter)
    }

    private val maneuverDistanceFormatter: DistanceFormatterOptions by lazy {
        DistanceFormatterOptions.Builder(this).build()
    }

    private val maneuverApi: MapboxManeuverApi by lazy {
        MapboxManeuverApi(MapboxDistanceFormatter(maneuverDistanceFormatter))
    }
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_navigation)

        carillonOptions = intent.getSerializableExtra("options") as Map<String, Objects>
        if (carillonOptions != null) {
            for (elem in carillonOptions["points"] as List<Map<String, Objects>>) {
                points.add(Point.fromLngLat(
                        elem["longitude"] as Double,
                        elem["latitude"] as Double
                ))
            }
        }

        tripProgressView = findViewById(R.id.tripProgressView)
        maneuverView = findViewById(R.id.maneuverView)
        maneuverView.visibility = INVISIBLE
        mapView = findViewById(R.id.mapView)
        overviewButton = findViewById(R.id.overview_button)
        recenterButton = findViewById(R.id.recenter_button)
        closeButton = findViewById(R.id.close_button)

        overviewButton.setOnClickListener {
            navigationCamera.requestNavigationCameraToOverview()
        }

        recenterButton.setOnClickListener {
            navigationCamera.requestNavigationCameraToFollowing()
        }

        closeButton.setOnClickListener {
            finish()
        }

        mapView.location.apply {
            this.locationPuck = LocationPuck2D(
                    bearingImage = ContextCompat.getDrawable(
                            this@NavigationActivity,
                            R.drawable.mapbox_navigation_puck_icon
                    )
            )
            setLocationProvider(navigationLocationProvider)
            enabled = true
        }

        viewportDataSource = MapboxNavigationViewportDataSource(mapView.getMapboxMap())
        navigationCamera = NavigationCamera(mapView.getMapboxMap(), mapView.camera, viewportDataSource)

        viewportDataSource.overviewPadding = overviewPadding
        viewportDataSource.followingPadding = followingPadding

        val accessToken = this.applicationContext.getString(this.applicationContext.resources.getIdentifier("mapbox_access_token", "string", this.applicationContext.packageName))

        val navigationOptions = NavigationOptions.Builder(this.applicationContext)
                .accessToken(accessToken)
                .build()
        val routeOptions = RouteOptions.builder()
                .applyDefaultNavigationOptions()
                .voiceUnits(DirectionsCriteria.METRIC)
                .coordinatesList(points.toList())
                .build()

        val mapboxRouteLineOptions = MapboxRouteLineOptions.Builder(this)
                .withRouteLineBelowLayerId("road-label")
                .build()

        routeLineApi = MapboxRouteLineApi(mapboxRouteLineOptions)
        routeLineView = MapboxRouteLineView(mapboxRouteLineOptions)

        routeArrowApi = MapboxRouteArrowApi()
        routeArrowOptions = RouteArrowOptions.Builder(this).build()
        routeArrowView = MapboxRouteArrowView(routeArrowOptions)

        mapView.camera.addCameraAnimationsLifecycleListener(
                NavigationBasicGesturesHandler(navigationCamera)
        )

        mapboxNavigation = MapboxNavigationProvider.create(navigationOptions)
        mapboxNavigation.startTripSession(withForegroundService = false)
        checkPermission()

        if (carillonOptions["mapStyle"] as String == "light")
            mapView.getMapboxMap().loadStyleUri(Style.LIGHT)
        else if (carillonOptions["mapStyle"] as String == "dark")
            mapView.getMapboxMap().loadStyleUri(Style.DARK)
        else
            mapView.getMapboxMap().loadStyleUri(Style.LIGHT)

        mapboxNavigation.requestRoutes(routeOptions, routeRequestCallBack)
    }

    private val routeRequestCallBack = object : RouterCallback {
        override fun onRoutesReady(routes: List<DirectionsRoute>, routerOrigin: RouterOrigin) {
            mapboxNavigation.setRoutes(routes)
        }

        override fun onCanceled(routeOptions: RouteOptions, routerOrigin: RouterOrigin) {
            TODO("Not yet implemented")
        }

        override fun onFailure(reasons: List<RouterFailure>, routeOptions: RouteOptions) {
            TODO("Not yet implemented")
        }
    }

    private val locationObserver = object : LocationObserver {
        var firstLocationUpdateReceived = false

        override fun onNewRawLocation(rawLocation: Location) {
            viewportDataSource.onLocationChanged(rawLocation)
            viewportDataSource.evaluate()
        }

        override fun onNewLocationMatcherResult(locationMatcherResult: LocationMatcherResult) {
            navigationLocationProvider.changePosition(
                    location = locationMatcherResult.enhancedLocation,
                    keyPoints = locationMatcherResult.keyPoints
            )
            viewportDataSource.onLocationChanged(locationMatcherResult.enhancedLocation)
            viewportDataSource.evaluate()

            if (!firstLocationUpdateReceived) {
                firstLocationUpdateReceived = true
                navigationCamera.requestNavigationCameraToFollowing(
                        stateTransitionOptions = NavigationCameraTransitionOptions.Builder()
                                .maxDuration(0) // instant transition
                                .build()
                )
            }
        }

    }

    private val routeObserver = RoutesObserver { result: RoutesUpdatedResult ->
        if (result.routes.isNotEmpty()) {
            val routeLines = result.routes.map { RouteLine(it, null) }
            routeLineApi.setRoutes(
                    routeLines
            ) { value ->
                mapView.getMapboxMap().getStyle()?.apply {
                    routeLineView.renderRouteDrawData(this, value)
                }
            }

            viewportDataSource.onRouteChanged(result.routes.first())
            viewportDataSource.evaluate()
        } else {
            val style = mapView.getMapboxMap().getStyle()
            if (style != null) {
                routeLineApi.clearRouteLine { value ->
                    routeLineView.renderClearRouteLineValue(
                            style,
                            value
                    )
                }
            }

            viewportDataSource.clearRouteData()
            viewportDataSource.evaluate()
        }
    }

    private val routeProgressObserver = RouteProgressObserver { routeProgress ->
        viewportDataSource.onRouteProgressChanged(routeProgress)
        viewportDataSource.evaluate()

        tripProgressApi.getTripProgress(routeProgress).let { update ->
            tripProgressView.render(update)
        }

        val maneuvers = maneuverApi.getManeuvers(routeProgress)
        maneuverView.renderManeuvers(maneuvers)
        maneuvers.fold(
                {
                    error ->
                },
                {
                    maneuverList ->
                        maneuverApi.getRoadShields(maneuverList, object : RoadShieldCallback {
                            override fun onRoadShields(maneuvers: List<Maneuver>,
                                                       shields: Map<String, RoadShield?>,
                                                       errors: Map<String, RoadShieldError>) {
                                maneuverView.renderManeuverShields(shields)
                                maneuverView.visibility = VISIBLE
                            }
                        })
                }
        )

        val updatedManeuverArrow = routeArrowApi.addUpcomingManeuverArrow(routeProgress)
        routeArrowView.renderManeuverUpdate(mapView.getMapboxMap().getStyle()!!, updatedManeuverArrow)

    }

    private val navigationCameraStateObserver = NavigationCameraStateChangedObserver { navigationCameraState ->
        when (navigationCameraState) {
            NavigationCameraState.TRANSITION_TO_FOLLOWING,
            NavigationCameraState.FOLLOWING -> recenterButton.visibility = GONE
            NavigationCameraState.TRANSITION_TO_OVERVIEW,
            NavigationCameraState.OVERVIEW,
            NavigationCameraState.IDLE -> recenterButton.visibility = VISIBLE
        }
    }

    private val arrivalObserver = object : ArrivalObserver {
        override fun onWaypointArrival(routeProgress: RouteProgress) {

        }
        override fun onNextRouteLegStart(routeLegProgress: RouteLegProgress) {

        }
        override fun onFinalDestinationArrival(routeProgress: RouteProgress) {
            finish() // to custom
        }
    }

    private fun checkPermission() {
        val hasperm = this.checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION)
        if (hasperm != PackageManager.PERMISSION_GRANTED) {
            this.requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), PERMISSION_REQUEST_CODE)
        }
    }

    override fun onStart() {
        super.onStart()
        mapboxNavigation.registerRouteProgressObserver(routeProgressObserver)
        mapboxNavigation.registerRoutesObserver(routeObserver)
        mapboxNavigation.registerLocationObserver(locationObserver)
        mapboxNavigation.registerArrivalObserver(arrivalObserver)
        navigationCamera.registerNavigationCameraStateChangeObserver(navigationCameraStateObserver)
    }

    override fun onStop() {
        super.onStop()
        mapboxNavigation.unregisterRoutesObserver(routeObserver)
        mapboxNavigation.unregisterRouteProgressObserver(routeProgressObserver)
        mapboxNavigation.unregisterLocationObserver(locationObserver)
        mapboxNavigation.unregisterArrivalObserver(arrivalObserver)
        navigationCamera.unregisterNavigationCameraStateChangeObserver(navigationCameraStateObserver)
    }

    override fun onDestroy() {
        super.onDestroy()
        routeLineApi.cancel()
        routeLineView.cancel()
        MapboxNavigationProvider.destroy()
    }

    private val pixelDensity = Resources.getSystem().displayMetrics.density
    private val overviewPadding: EdgeInsets by lazy {
        EdgeInsets(
                140.0 * pixelDensity,
                40.0 * pixelDensity,
                120.0 * pixelDensity,
                40.0 * pixelDensity
        )
    }
    private val followingPadding: EdgeInsets by lazy {
        EdgeInsets(
                180.0 * pixelDensity,
                40.0 * pixelDensity,
                150.0 * pixelDensity,
                40.0 * pixelDensity
        )
    }
}
